@@ -105,6 +105,11 @@ public:
 
   static sharedptr<SharedConnection> get_and_connect();
 
+  /** This specifies that Glom should start its own database server instance for this database,
+   *  using the database files stored at the specified uri.
+   */
+  void set_self_hosted(const std::string& data_uri);
+
 
   void set_host(const Glib::ustring& value);
   void set_user(const Glib::ustring& value);
@@ -125,18 +130,35 @@ public:
    */
   float get_postgres_server_version();
 
+  /** Start a database server instance for the exisiting database files.
+   */
+  void start_self_hosting();
+
+  /** Stop the database server instance for the database files.
+   */
+  void stop_self_hosting();
+
+  /** Create new database files, for later use by their own  database server instance.
+   */
+  bool create_self_hosting();
+
   //Show the gda error in a dialog.
   static bool handle_error(bool cerr_only = false);
 
 protected:
   void on_sharedconnection_finished();
 
+  static bool create_text_file(const std::string& file_uri, const std::string& contents);
+
+
   typedef std::list<Glib::ustring> type_list_ports;
   type_list_ports m_list_ports; //Network ports on which to try connecting to postgres.
 
   Glib::RefPtr<Gnome::Gda::Client> m_GdaClient;
   //Gnome::Gda::DataSourceInfo m_GdaDataSourceInfo;
 
+  bool m_self_hosting_active;
+  std::string m_self_hosting_data_uri;
 
   Glib::RefPtr<Gnome::Gda::Connection> m_refGdaConnection;
   guint m_sharedconnection_refcount;
