@@ -95,13 +95,29 @@ public:
 
   static guint get_latest_known_document_format_version();
 
-  virtual void set_connection_server(const Glib::ustring& strVal);
-  virtual void set_connection_user(const Glib::ustring& strVal);
-  virtual void set_connection_database(const Glib::ustring& strVal);
+  /** When this is set, the postgres database is hosted by the local client,
+   *  instead of connecting to an external database server.
+   */
+  void set_connection_is_self_hosted(bool self_hosted = true);
 
-  virtual Glib::ustring get_connection_server() const;
-  virtual Glib::ustring get_connection_user() const;
-  virtual Glib::ustring get_connection_database() const;
+  void set_connection_server(const Glib::ustring& strVal);
+  void set_connection_user(const Glib::ustring& strVal);
+  void set_connection_database(const Glib::ustring& strVal);
+
+  /** When this returns true, the postgres database should be hosted by the local client,
+   *  instead of connecting to an external database server.
+   */
+  bool get_connection_is_self_hosted() const;
+
+  /** If the database should be hosted, this provides the 
+    * path to the directory that contains all the files needed to do that.
+    * This is usually a specifically-named directory at the same level as the .glom file. 
+    */   
+  std::string get_connection_self_hosted_directory_uri() const;
+
+  Glib::ustring get_connection_server() const;
+  Glib::ustring get_connection_user() const;
+  Glib::ustring get_connection_database() const;
 
   /** Set the language/locale used by original titles.
    * Title translations are translations of the text in this language.
@@ -115,7 +131,7 @@ public:
   Glib::ustring get_translation_original_locale() const;
 
   typedef std::vector< sharedptr<Relationship> > type_vecRelationships;
-  virtual type_vecRelationships get_relationships(const Glib::ustring& table_name, bool plus_system_prefs = false) const;
+  type_vecRelationships get_relationships(const Glib::ustring& table_name, bool plus_system_prefs = false) const;
   void set_relationships(const Glib::ustring& table_name, const type_vecRelationships& vecRelationships);
 
   sharedptr<Relationship> get_relationship(const Glib::ustring& table_name, const Glib::ustring& relationship_name) const;
@@ -135,10 +151,10 @@ public:
 
 
   typedef std::vector< sharedptr<Field> > type_vecFields;
-  virtual type_vecFields get_table_fields(const Glib::ustring& table_name) const;
-  virtual void set_table_fields(const Glib::ustring& table_name, const type_vecFields& vecFields);
+  type_vecFields get_table_fields(const Glib::ustring& table_name) const;
+  void set_table_fields(const Glib::ustring& table_name, const type_vecFields& vecFields);
 
-  virtual sharedptr<Field> get_field(const Glib::ustring& table_name, const Glib::ustring& strFieldName) const;
+  sharedptr<Field> get_field(const Glib::ustring& table_name, const Glib::ustring& strFieldName) const;
 
   /** Use this after removing a field from a table,
    * so that it is not used anymore in relationships, layouts, reports, etc.
@@ -178,7 +194,7 @@ public:
   void change_relationship_name(const Glib::ustring& table_name, const Glib::ustring& name, const Glib::ustring& name_new);
 
   typedef std::list< sharedptr<TableInfo> > type_listTableInfo;
-  virtual type_listTableInfo get_tables(bool plus_system_prefs = false) const;
+  type_listTableInfo get_tables(bool plus_system_prefs = false) const;
   std::vector<Glib::ustring> get_table_names(bool plus_system_prefs = false) const;
 
   void set_tables(const type_listTableInfo& tables);
@@ -190,13 +206,13 @@ public:
    * It removes information about the table, including fields and layouts,
    * and any place that parts of the table are used.
    */
-  virtual void remove_table(const Glib::ustring& table_name);
+  void remove_table(const Glib::ustring& table_name);
 
-  virtual bool get_table_is_known(const Glib::ustring& table_name) const;
-  virtual bool get_table_is_hidden(const Glib::ustring& table_name) const;
+  bool get_table_is_known(const Glib::ustring& table_name) const;
+  bool get_table_is_hidden(const Glib::ustring& table_name) const;
 
-  virtual Glib::ustring get_table_title(const Glib::ustring& table_name) const;
-  virtual void set_table_title(const Glib::ustring& table_name, const Glib::ustring& value);
+  Glib::ustring get_table_title(const Glib::ustring& table_name) const;
+  void set_table_title(const Glib::ustring& table_name, const Glib::ustring& value);
 
   /** Save example data into the document, for use when creating the example database on the server.
    * Don't use this for large amounts of data.
@@ -209,11 +225,16 @@ public:
 
   virtual Glib::ustring get_name() const; //override.
 
-  virtual Glib::ustring get_default_table() const;
-  virtual Glib::ustring get_first_table() const;
+  Glib::ustring get_default_table() const;
+  Glib::ustring get_first_table() const;
 
-  virtual Glib::ustring get_database_title() const;
-  virtual void set_database_title(const Glib::ustring& title);
+  Glib::ustring get_database_title() const;
+  void set_database_title(const Glib::ustring& title);
+
+  std::vector<Glib::ustring> get_library_module_names() const;
+  void set_library_module(const Glib::ustring& name, const Glib::ustring& script);
+  Glib::ustring get_library_module(const Glib::ustring& name) const;
+  void remove_library_module(const Glib::ustring& name);
 
   /// These are only used when recreating a database from an example file. The actualy access-control is on the server, of course.
   typedef std::list<GroupInfo> type_list_groups;
@@ -251,18 +272,18 @@ public:
    * @param reason The reason that the user is not a developer, if he is not.
    * @result Whether the user is a developer.
    */
-  virtual AppState::userlevels get_userlevel(userLevelReason& reason) const;
+  AppState::userlevels get_userlevel(userLevelReason& reason) const;
 
-  virtual AppState::userlevels get_userlevel() const;
+   AppState::userlevels get_userlevel() const;
 
   /** This is transitory information, not saved to disk.
    */
-  virtual bool set_userlevel(AppState::userlevels userlevel);
+  bool set_userlevel(AppState::userlevels userlevel);
 
   typedef sigc::signal<void, AppState::userlevels> type_signal_userlevel_changed;
   type_signal_userlevel_changed signal_userlevel_changed();
 
-  virtual void emit_userlevel_changed();
+  void emit_userlevel_changed();
 
   void set_parent_window(Gtk::Window* window);
 
@@ -302,7 +323,7 @@ protected:
 
   void save_changes();
 
-  virtual void on_app_state_userlevel_changed(AppState::userlevels userlevel);
+  void on_app_state_userlevel_changed(AppState::userlevels userlevel);
 
   void fill_translatable_layout_items(const sharedptr<LayoutGroup>& group, type_list_translatables& the_list);
 
@@ -318,9 +339,12 @@ protected:
   Glib::ustring get_child_text_node(const xmlpp::Element* node, const Glib::ustring& child_node_name) const;
   void set_child_text_node(xmlpp::Element* node, const Glib::ustring& child_node_name, const Glib::ustring& text);
 
+  void append_newline(xmlpp::Element* parent_node);
+
   AppState m_app_state;
   type_signal_userlevel_changed m_signal_userlevel_changed;
 
+  bool m_connection_is_self_hosted;
   Glib::ustring m_connection_server, m_connection_user, m_connection_database;
 
   class LayoutInfo
@@ -373,6 +397,9 @@ protected:
   Glib::ustring m_database_title;
   Glib::ustring m_translation_original_locale;
 
+  typedef std::map<Glib::ustring, Glib::ustring> type_map_library_scripts;
+  type_map_library_scripts m_map_library_scripts;
+
   bool m_block_cache_update; //For efficiency.
   bool m_block_modified_set;
   bool m_allow_auto_save;
