@@ -24,6 +24,8 @@
 #include "../box_db_table.h"
 #include "dialog_layout.h"
 
+namespace Glom
+{
 
 /** Call init_db_details() to create the layout and fill it with data from the database.
  * Call refresh_data_from_database() to fill the existing layout with up-to-date data from the database.
@@ -38,15 +40,15 @@ public:
   virtual ~Box_Data();
 
   ///Create the layout for the database structure, and fill it with data from the database.
-  virtual bool init_db_details(const Glib::ustring& table_name, const Glib::ustring& where_clause = Glib::ustring());
+  virtual bool init_db_details(const FoundSet& found_set);
 
   //Fill the existing layout with data from the database:
-  virtual bool refresh_data_from_database_with_where_clause(const Glib::ustring& where_clause = Glib::ustring());
+  virtual bool refresh_data_from_database_with_where_clause(const FoundSet& found_set);
 
   virtual void print_layout(); //A test, for now.
 
   ///Get the existing where clause, previously supplied to init_db_details().
-  Glib::ustring get_where_clause() const;
+  FoundSet get_found_set() const;
 
   virtual Glib::ustring get_find_where_clause() const;
 
@@ -83,10 +85,10 @@ protected:
   ///Fill the existing layout with data from the database.
   virtual bool fill_from_database(); //override.
 
-  virtual void refresh_related_fields(const FieldInRecord& field_in_record_changed, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value);
+  virtual void refresh_related_fields(const LayoutFieldInRecord& field_in_record_changed, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value);
 
   virtual type_vecLayoutFields get_fields_to_show() const;
-  //virtual Glib::ustring build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause);
+
   virtual bool get_related_record_exists(const sharedptr<const Relationship>& relationship, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value);
   virtual bool add_related_record_for_field(const sharedptr<const LayoutItem_Field>& layout_item_parent, const sharedptr<const Relationship>& relationship, const sharedptr<const Field>& primary_key_field, const Gnome::Gda::Value& primary_key_value_provided);
 
@@ -100,10 +102,17 @@ protected:
 
   /** Get the fields that are in related tables, via a relationship using @a field_name changes.
   */
-  type_vecLayoutFields get_related_fields(const Glib::ustring& field_name) const;
+  type_vecLayoutFields get_related_fields(const sharedptr<const LayoutItem_Field>& field) const;
+
+  bool record_delete(const Gnome::Gda::Value& primary_key_value);
+
+  ///This allows record_new() to set the generated/entered primary key value, needed by Box_Data_List:
+  virtual void set_primary_key_value(const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& value);
+
+  ///New record with all entered field values.
+  Glib::RefPtr<Gnome::Gda::DataModel> record_new(bool use_entered_data = true, const Gnome::Gda::Value& primary_key_value = Gnome::Gda::Value()); 
+  Glib::RefPtr<Gnome::Gda::DataModel> record_new(bool use_entered_data, const Gnome::Gda::Value& primary_key_value, const Gtk::TreeModel::iterator& row);
 
-  virtual bool record_delete(const Gnome::Gda::Value& primary_key_value);
-  virtual Glib::RefPtr<Gnome::Gda::DataModel> record_new(bool use_entered_data = true, const Gnome::Gda::Value& primary_key_value = Gnome::Gda::Value()); //New record with all entered field values.
   Gnome::Gda::Value generate_next_auto_increment(const Glib::ustring& table_name, const Glib::ustring field_name);
 
   virtual sharedptr<Field> get_field_primary_key() const = 0;
@@ -125,10 +134,12 @@ protected:
   Dialog_Layout* m_pDialogLayout;
   Glib::ustring m_layout_name;
 
-  Glib::ustring m_where_clause;
+  FoundSet m_found_set;
 
   type_vecFields m_TableFields; //A cache, so we don't have to repeatedly get them from the Document.
   type_vecLayoutFields m_FieldsShown; //And any extra keys needed by shown fields.
 };
 
+} //namespace Glom
+
 #endif
